import { LogMessage, LogLevel } from './log-message';
import LogAppender from './log-appender';

export default class Logger {

  public static readonly ROOT_LOGGER = new Logger('ROOT', undefined, LogLevel.ALL);

  public appenders: LogAppender[] = [];

  protected constructor(public readonly name: string, private parentLogger?: Logger, public threshold?: LogLevel) { }

  public createChildLogger(name: string, threshold?: LogLevel): Logger {
    return new Logger(name, this, threshold);
  }

  public log(level: LogLevel, message: (Error | string), stacktrace?: string): void {

    if (level < this.getEffectiveThreshold()) { return; }

    const logMsg: LogMessage = { level, message: '', stacktrace: '', timestamp: new Date() };

    if (typeof message === 'string') {
      logMsg.message = message;
      logMsg.stacktrace = stacktrace == null ? '' : stacktrace;
    } else {
      logMsg.error = message;
      logMsg.message = `${message.name}: ${message.message}`;
      logMsg.stacktrace = message.stack == null ? '' : message.stack;
    }

    this.appenders.forEach((app) => {
      app.appendMessage(this, logMsg);
    });
  }

  public debug(message: (Error | string), stacktrace?: string): void {
    this.log(LogLevel.DEBUG, message, stacktrace);
  }

  public info(message: string, stacktrace?: string): void {
    this.log(LogLevel.INFO, message, stacktrace);
  }

  public warn(message: string, stacktrace?: string): void {
    this.log(LogLevel.WARN, message, stacktrace);
  }

  public error(message: string, stacktrace?: string): void {
    this.log(LogLevel.ERROR, message, stacktrace);
  }

  public fatal(message: string, stacktrace?: string): void {
    this.log(LogLevel.FATAL, message, stacktrace);
  }

  protected getEffectiveThreshold(): LogLevel {
    if (this.threshold) { return this.threshold; }
    if (this.parentLogger) { return this.parentLogger.getEffectiveThreshold(); }

    // should never happen (root logger should always have a threshold
    return LogLevel.ALL;
  }

}


